#!/bin/bash
#
# maclinuxdevelopment@pandasecurity.com - 2017-2018
#
# 0.1.0 - 25/10/2018
# 0.0.9 - 04/09/2018
# 0.0.8 - 28/05/2018
# 0.0.1 - first release (22/01/2018)
#

OS=$(uname -s)
ID=$(id -u)
[ -d "/usr/local/protection-agent" ] || [ -d "/Applications/Protection-Agent.app" ] && PA_INSTALLED=1 || PA_INSTALLED=0
[ -d "/usr/local/management-agent" ] || [ -d "/Applications/Management-Agent.app" ] && MA_INSTALLED=1 || MA_INSTALLED=0
INSTALL_DIR="/usr/local/protection-agent"

TOOL_FEDORA=./ua-tool-fedora
TOOL_UBUNTU=./ua-tool-ubuntu
TOOL_OSX=./ua-tool-osx
UA_TOOL=$TOOL_UBUNTU
LOGS_DIR="aether-logs-$(/bin/date +%Y%m%d-%H%M%S/)$OS"
CORES_DIR="$LOGS_DIR/cores"
MD5SUM="md5sum"
URLS="8.8.8.8 www.google.es cpp-kw.pandasecurity.com cpp-fusm.pandasecurity.com iext.pandasecurity.com"
CURL_AVAILABLE=$(curl -V &>/dev/null; echo $?)

function warning_app_not_found 
{
    echo "[!] $1 not found, skipping"
}

function check_product_installed 
{
    if [ ! -d $INSTALL_DIR ]
    then
        echo "[!!] $INSTALL_DIR does not exist, is the product installed?"
        echo
        return
    fi
}

function gather_linux_logs 
{
#        FLAVOUR=$(lsb_release -s -i &>/dev/null || cut -d " " -f 1 /etc/fedora-release)
        echo "[+] Starting gathering logs in GNU/Linux"
        echo "[+] Linux flavour: $FLAVOUR"
        echo '' > $LOGS_DIR/PA_files_ldd.txt
        find $INSTALL_DIR -type f -executable -exec echo {} >> $LOGS_DIR/PA_files_ldd.txt \; -exec ldd {} >> $LOGS_DIR/PA_files_ldd.txt \;
        echo "[+] gathering system stats, wait..."
        sudo top -n 10 >> $LOGS_DIR/top-10.txt
        echo "[+] continuing..."
        sudo /bin/ps fuxa > $LOGS_DIR/ps.txt
        sudo netstat -pa > $LOGS_DIR/netstat.txt
        free -m > $LOGS_DIR/free.txt
        lsmod > $LOGS_DIR/lsmod.txt
        iostat -d 1 10 >> $LOGS_DIR/iostat-10.txt 2>/dev/null || warning_app_not_found "iostat"
        echo "[+] we're not dead yet..."
        sudo ip r > $LOGS_DIR/routes.txt || sudo route -n > $LOGS_DIR/routes.txt || /bin/cat /proc/net/route > $LOGS_DIR/routes.txt
        sudo iptables -t nat -L -v > $LOGS_DIR/iptables.txt
        sudo iptables -L -v >> $LOGS_DIR/iptables.txt
        sudo lshw 2>&1 > $LOGS_DIR/lshw.txt 2>/dev/null || warning_app_not_found "lshw"
        vmstat -wn 1 10 >> $LOGS_DIR/vmstat.txt 2>/dev/null || warning_app_not_found "vmstat"

        if [ -x /usr/bin/journalctl ]; then
            sudo journalctl -ar > $LOGS_DIR/journalctl.txt
        fi
        if [ -d /var/spool/abrt ]
        then
            for i in $(sudo find /var/spool/abrt/ -name 'cmdline' -exec echo {} \; -exec grep -iE "(pa_service|pavapi_daemon|squid|pa_acl_helpers|pa_gui)" {} \;)
            do
                if [ -f $i ]
                then
                    sudo /bin/cp -a $(dirname $i) $CORES_DIR
                fi
            done
        fi
        if [ "$FLAVOUR" = "Ubuntu" ]; then
            sudo dpkg -l > $LOGS_DIR/dpkg.txt
        fi

        if [ $MA_INSTALLED -eq 1 ]
        then
            mkdir -p $LOGS_DIR/ma/
            $MD5SUM $MA_DIR/downloads/* > $LOGS_DIR/ma/downloads_ms5.txt
            $MD5SUM $MA_DIR/downloads/KnowledgeFiles/* >> $LOGS_DIR/ma/downloads_ms5.txt
            ls -lR $MA_DIR/downloads/* >> $LOGS_DIR/ma/downloads_ms5.txt
            cp -a $MA_DIR/downloads/KnowledgeFiles/ $LOGS_DIR/ma/
        fi
	
        for f in $(seq 0 ${#LOGS[*]})
        do
            if [ "${LOGS[$f]}" == " " ]
            then
               echo "skipping copy"
               continue
            fi
            echo "[+] Copying ${LOGS[$f]} to $LOGS_DIR/"
            sudo /bin/cp -a "${LOGS[$f]}" $LOGS_DIR/ &>/dev/null
        done
        
        cat /etc/os-release > $LOGS_DIR/os-release
}

function gather_mac_logs
{
        LOGS=(/var/log/system.log /var/log/install.log /etc/hosts /Library/Preferences/SystemConfiguration/*)
        PA_LOGS=(/Applications/Protection-Agent.app/Contents/Info.plist  "/Library/Application Support/Protection Agent/quarantine" "/Library/Application Support/Protection Agent/etc" "/Library/Application Support/Protection Agent/log")
        MA_LOGS=(/Applications/Management-Agent.app/Contents/etc /Applications/Management-Agent.app/Contents/Info.plist "/Library/Application Support/Management Agent/log")
        MD5SUM="md5"
        echo "[+] Starting gathering logs in OSX"
        sudo cp -a /Library/Logs/DiagnosticReports/* $CORES_DIR
        sudo /bin/cp /Library/Application\ Support/Protection\ Agent/etc/pa_service.info $LOGS_DIR/
        sudo top -l 10 >> $LOGS_DIR/top-10.txt
        sudo /bin/ps -faxe > $LOGS_DIR/ps.txt
        sudo netstat -a > $LOGS_DIR/netstat.txt
        sudo netstat -nr > $LOGS_DIR/routes.txt
        [ -x /usr/bin/iostat ] && iostat -c 10 >> $LOGS_DIR/iostat-10.txt
        sudo pfctl -vs all > $LOGS_DIR/pfctl.txt
        sudo pfctl -a panda_firewall -vsn >> $LOGS_DIR/pfctl.txt

        for f in $(seq 0 ${#LOGS[*]})
        do
            echo "[+] Copying ${LOGS[$f]} to $LOGS_DIR/"
            sudo /bin/cp -a "${LOGS[$f]}" $LOGS_DIR/ &>/dev/null
        done
        
        if [ $MA_INSTALLED -eq 1 ]
        then
            mkdir -p $LOGS_DIR/ma
            MA_DL_DIR="/Library/Application Support/Management Agent/downloads"
            $MD5SUM "$MA_DL_DIR"/* > $LOGS_DIR/ma/downloads_md5.txt
            ls -lR "$MA_DL_DIR"/* >> $LOGS_DIR/ma/downloads_md5.txt
            $MD5SUM "$MA_DL_DIR"/KnowledgeFiles/* >> $LOGS_DIR/ma/downloads_md5.txt
            cp -a "$MA_DL_DIR"/KnowledgeFiles $LOGS_DIR/ma/
        fi
	
        echo "[+] Running sysdiagnose, this is gonna take a while..."
        sysdiagnose -b -f $LOGS_DIR/ -A sysdiagnose
}

function gather_pa_logs
{
    # copy product and system logs
    if [ $PA_INSTALLED -eq 1 ]
    then
        mkdir -p $LOGS_DIR/pa/
        for f in $(seq 0 ${#PA_LOGS[*]})
        do
            echo "[+] Copying ${PA_LOGS[$f]} to $LOGS_DIR/pa/"
            sudo /bin/cp -a "${PA_LOGS[$f]}" $LOGS_DIR/pa/ &>/dev/null
        done
    else
        echo "[-] PA not installed, skipping"
    fi
}

function gather_ma_logs
{
    # copy MA logs and data
    if [ $MA_INSTALLED -eq 1 ]
    then
        mkdir -p $LOGS_DIR/ma/
        for f in $(seq 0 ${#MA_LOGS[*]})
        do
            echo "[+] Copying ${MA_LOGS[$f]} to $LOGS_DIR/ma/"
            sudo /bin/cp -a "${MA_LOGS[$f]}" $LOGS_DIR/ma/ &>/dev/null
        done
    else
        echo "[-] MA not installed, skipping"
    fi
}

function gather_logs
{
    check_product_installed

    LOGS=(/var/log/syslog /var/log/messages /var/log/kern.log /var/log/debug /var/log/daemon /etc/hosts /etc/hostname /etc/lsb-release)
    PA_LOGS=($INSTALL_DIR/log/ $INSTALL_DIR/etc/ $INSTALL_DIR/squid/var/logs/* $INSTALL_DIR/quarantine/)
    MA_LOGS=($MA_DIR/log/ $MA_DIR/etc/)
    mkdir -p $LOGS_DIR || (echo "logs directory not created, check /tmp/ perms" && exit)
    mkdir -p $CORES_DIR

    if [ "$OS" = "Linux" ] ; then
        gather_linux_logs
    else
        gather_mac_logs
    fi

    gather_pa_logs
    gather_ma_logs

    uname -a > $LOGS_DIR/uname.txt

    if [[ -f $UA_TOOL ]]
    then
        $UA_TOOL --info > $LOGS_DIR/uatool_info.txt
        $UA_TOOL --status > $LOGS_DIR/uatool_status.txt
        $UA_TOOL --license > $LOGS_DIR/uatool_license_info.txt
        $UA_TOOL --license --check > $LOGS_DIR/uatool_license_status.txt
        $UA_TOOL --update --showsigfilesinfo > $LOGS_DIR/uatool_sigfilesinfo.txt
        $UA_TOOL --report --showall > $LOGS_DIR/uatool_reports.txt
        $UA_TOOL --quarantine --showall > $LOGS_DIR/uatool_quarantine.txt
    else
        echo "[!] $UA_TOOL not found. Debug logs can't be activated and we are unable to gather product health status"
    fi

    PID=$(pgrep pa_service) 
    [ "$PID" != "" ] && sudo lsof -a -p $PID 2>&1 > $LOGS_DIR/lsof_paservice.txt
    PID=$(pgrep squid) 
    [ "$PID" != "" ] && sudo lsof -a -p $PID 2>&1 > $LOGS_DIR/lsof_squid.txt
    PID=$(pgrep PavapiDaemon) 
    [ "$PID" != "" ] && sudo lsof -a -p $PID 2>&1 > $LOGS_DIR/lsof_pavapidaemon.txt

    echo "[+] almost finished..."
           
    # get product health info
    sudo /bin/ls -lR $INSTALL_DIR > $LOGS_DIR/PA_files_tree.log
        
    sudo /bin/cat $INSTALL_DIR/pavapi/daemon/.pid/.pavapi-daemon.pid > $LOGS_DIR/pavapidaemon_pid.txt 2>/dev/null
    [ "$?" != "0" ] && echo "[!] PavapiDaemon seems to be not running. REVIEW"
    sudo /bin/cat $INSTALL_DIR/squid/var/run/squid.pid > $LOGS_DIR/squid_pid.txt 2>/dev/null
    [ "$?" != "0" ] && echo "[!] squid seems to be not running. REVIEW"

    # pavsigs md5s and size
    $MD5SUM $INSTALL_DIR/pavapi/pavapi/opts/* > $LOGS_DIR/pavapi_sigs.txt
    echo "-----" >> $LOGS_DIR/pavapi_sigs.txt
    $MD5SUM $INSTALL_DIR/pavapi/pavapi/sigs/* >> $LOGS_DIR/pavapi_sigs.txt &>/dev/null

    sudo dmesg > $LOGS_DIR/dmesg.txt
    df -h > $LOGS_DIR/df.txt
    sudo sysctl -a > $LOGS_DIR/sysctl.txt 2>/dev/null
    sudo ifconfig -a > $LOGS_DIR/ifconfig.txt
    /bin/cat /etc/resolv.conf > $LOGS_DIR/resolv.txt
    ulimit -a > $LOGS_DIR/ulimit.txt
    mount > $LOGS_DIR/mount.txt || warning_app_not_found "mount"
    for u in $URLS
    do
        ping -c 5 $u 2>&1 >> $LOGS_DIR/net_connectivity.txt
    done
   
    sudo chmod -R a+r $LOGS_DIR
    sudo chown -R $(id -u -n) $LOGS_DIR
    TARFILE="panda_logs_$(date +%Y%m%d%H%M%S).tar.gz"
    tar zcf $TARFILE $LOGS_DIR/ || echo "[!!] targz archive not created. Please, compress it by hand"
    /bin/rm -rf $LOGS_DIR
    /bin/rmdir $(echo $LOGS_DIR | cut -d/ -f1)
    echo
    echo
    echo "[+] DONE!"
    echo 
    echo "[+] The logs are at $TARFILE"
    echo
}

function reload_fw_rules
{
    echo "[i] Reloading firewall rules, wait... "
    if [ "$OS" = "Darwin" ] ; then
        sudo pfctl -a panda_firewall -F nat 2>/dev/null
        sudo pfctl -f /etc/pf.conf 2>/dev/null
    else
        sudo iptables -t nat -F OUTPUT
    fi

    tries=10
    while sleep 1
    do
        test_firewall_rules
        if [ "$?" -eq 0 ]
        then
           echo "OK"
           break
        fi
        printf "[i] tries left $tries   \r"
        if [  $tries -lt 1  ]
        then
           echo "[!!] Firewall rules not reloaded. Check out the pa_service process"
           echo "[!] FAIL"
           break
        fi
        tries=$(($tries - 1))
    done
}

function test_firewall_rules
{
    if [ "$OS" = "Darwin" ] ; then
        pfctl -a panda_firewall -sn 2>/dev/null | grep 312 >/dev/null
    else
        iptables -t nat -L OUTPUT | grep "SquidProxy" >/dev/null
    fi

    return $?
}

function test_URL_protection
{
    if [ ! $CURL_AVAILABLE -eq 0 ]
    then
        echo "[!] curl not installed, not possible to perform URL protection test. Install it and try again"
        return
    fi

    echo "[*] Starting URL protection test"

    test_firewall_rules
    if [ $? = 1 ]
    then
        echo "[!!] The firewall rules are not set. Review them."
    else
        echo "[+] Firewall rules are set"
    fi

    $UA_TOOL --setloglevel --area ALL --level TRACE >/dev/null
    sleep 1
    echo "[*] Testing Malware URL"
    curl --connect-timeout 5 -v --stderr - -L http://websectest.ctmail.com/42__Malware.htm | grep "\/\/weblock\.pandasecurity\.com\/" >/dev/null
    if [ $? -eq 0 ]
    then
        echo "[i] URL protection is working fine. If it keeps failing, check your browser, erase cache, reload the page, etc, "
    else
        echo "[!] UhOh, URL protection is not working"
        read -n 1 -p "[?] Reload firewall rules and try again? (y/n): " RELOAD_FW
        echo "\n"
        if [ $RELOAD_FW = y ]
        then
            reload_fw_rules
            test_URL_protection
            return
        fi
    fi
    $UA_TOOL --setloglevel --area ALL --level OFF >/dev/null
    gather_logs
}

function menu
{
    echo ""
    echo " Management Agent installed: $MA_INSTALLED"
    echo " Protection Agent installed: $PA_INSTALLED"
    echo ""
    echo " Options:"
    echo "  1. Activate debug logs"
    echo "  2. Deactivate debug logs"
    echo "  3. Gather system logs"
    echo "  4. Test URL protection"
    echo "  5. Exit"

    if [ $ID -ne 0 ]
    then
        echo "[!] WARNING: You need to be root to gather all the logs properly"
    fi

    if [ "$OS" = "Darwin" ]
    then
        echo
    else
        echo -n "> "
    fi

    read -n 2 REPLY
    if [ "$REPLY" = "1" ]
    then
        $UA_TOOL --setloglevel --area ALL --level DEBUG
        if [ $? -ne 0 ]
        then
            echo
            echo "WARNING: There's no communication with the PA service. You may need to restart the service"
            echo
        else
            echo
            echo "DONE!"
            echo "Now you can reproduce the problem."
        fi
    elif [ "$REPLY" = "2" ]
    then
        $UA_TOOL --setloglevel --area ALL --level OFF
        echo
        echo "DONE!"
        echo "Now please, gather the logs (option 1), and send them back to us"
    elif [ "$REPLY" = "3" ]
    then
        gather_logs
    elif [ "$REPLY" = "4" ]
    then
        test_URL_protection
    elif [ "$REPLY" = "5" ]
    then
        echo "bye..."
        exit
    else
        echo ">> Invalid option: $REPLY"
    fi
    menu 
}

###############################################################################
# main
###############################################################################

if [ "$OS" = "Darwin" ] ; then
    UA_TOOL=$TOOL_OSX
    INSTALL_DIR="/Applications/Protection-Agent.app/Contents"
    MA_DIR="/Applications/Management-Agent.app/Contents"
elif [ "$OS" = "Linux" ] ; then
    INSTALL_DIR=/usr/local/protection-agent
    MA_DIR=/usr/local/management-agent
    FLAVOUR=$(lsb_release -s -i 2>/dev/null || cut -d " " -f 1 /etc/fedora-release)
    if [ "$FLAVOUR" = "Fedora" ]
    then
        UA_TOOL=$TOOL_FEDORA
    elif [ "$FLAVOUR" = "Ubuntu" ]
    then
        UA_TOOL=$TOOL_UBUNTU
    fi
else
    echo "OS not supported? -> $OS"
    exit
fi

menu
